Package: simmer
Type: Package
Title: Discrete-Event Simulation for R
Version: 4.4.7
Authors@R: c(
    person("Iñaki", "Ucar", email="iucar@fedoraproject.org",
      role=c("aut", "cph", "cre"), comment=c(ORCID="0000-0001-6403-5550")),
    person("Bart", "Smeets", email="bartsmeets86@gmail.com", role=c("aut", "cph")))
Description: A process-oriented and trajectory-based Discrete-Event Simulation
    (DES) package for R. It is designed as a generic yet powerful framework. The
    architecture encloses a robust and fast simulation core written in 'C++' with
    automatic monitoring capabilities. It provides a rich and flexible R API that
    revolves around the concept of trajectory, a common path in the simulation
    model for entities of the same type. Documentation about 'simmer' is provided
    by several vignettes included in this package, via the paper by Ucar, Smeets
    & Azcorra (2019, <doi:10.18637/jss.v090.i02>), and the paper by Ucar,
    Hernández, Serrano & Azcorra (2018, <doi:10.1109/MCOM.2018.1700960>);
    see 'citation("simmer")' for details.
License: GPL (>= 2)
Encoding: UTF-8
URL: https://r-simmer.org, https://github.com/r-simmer/simmer
BugReports: https://github.com/r-simmer/simmer/issues
Depends: R (>= 3.4.0)
Imports: Rcpp, magrittr, codetools, utils
Suggests: simmer.plot, parallel, testthat, knitr, rmarkdown, rticles
LinkingTo: Rcpp (>= 0.12.9)
RoxygenNote: 7.3.2
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2024-09-27 15:53:49 UTC; iucar
Author: Iñaki Ucar [aut, cph, cre] (<https://orcid.org/0000-0001-6403-5550>),
  Bart Smeets [aut, cph]
Maintainer: Iñaki Ucar <iucar@fedoraproject.org>
Repository: CRAN
Date/Publication: 2024-09-28 00:10:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 22:21:52 UTC; unix
Archs: simmer.so.dSYM
