# Generated by roxygen2: do not edit by hand

S3method("+",schedule)
S3method("[",trajectory)
S3method("[<-",trajectory)
S3method("[[",trajectory)
S3method("[[<-",trajectory)
S3method(activate,trajectory)
S3method(add_dataframe,simmer)
S3method(add_generator,simmer)
S3method(add_global,simmer)
S3method(add_resource,simmer)
S3method(batch,trajectory)
S3method(branch,trajectory)
S3method(clone,trajectory)
S3method(deactivate,trajectory)
S3method(get_activity_time,simmer)
S3method(get_activity_time_selected,simmer)
S3method(get_attribute,simmer)
S3method(get_batch_size,simmer)
S3method(get_capacity,simmer)
S3method(get_capacity,wrap)
S3method(get_capacity_selected,simmer)
S3method(get_global,simmer)
S3method(get_mon_arrivals,simmer)
S3method(get_mon_arrivals,wrap)
S3method(get_mon_attributes,simmer)
S3method(get_mon_attributes,wrap)
S3method(get_mon_resources,simmer)
S3method(get_mon_resources,wrap)
S3method(get_n_activities,trajectory)
S3method(get_n_generated,simmer)
S3method(get_n_generated,wrap)
S3method(get_name,simmer)
S3method(get_prioritization,simmer)
S3method(get_queue_count,simmer)
S3method(get_queue_count,wrap)
S3method(get_queue_count_selected,simmer)
S3method(get_queue_size,simmer)
S3method(get_queue_size,wrap)
S3method(get_queue_size_selected,simmer)
S3method(get_resources,simmer)
S3method(get_resources,wrap)
S3method(get_seized,simmer)
S3method(get_seized_selected,simmer)
S3method(get_selected,simmer)
S3method(get_server_count,simmer)
S3method(get_server_count,wrap)
S3method(get_server_count_selected,simmer)
S3method(get_sources,simmer)
S3method(get_sources,wrap)
S3method(get_start_time,simmer)
S3method(get_trajectory,simmer)
S3method(handle_unfinished,trajectory)
S3method(join,trajectory)
S3method(leave,trajectory)
S3method(length,trajectory)
S3method(log_,trajectory)
S3method(now,simmer)
S3method(now,wrap)
S3method(peek,simmer)
S3method(peek,wrap)
S3method(print,monitor)
S3method(print,schedule)
S3method(print,simmer)
S3method(print,trajectory)
S3method(print,wrap)
S3method(release,trajectory)
S3method(release_all,trajectory)
S3method(release_selected,trajectory)
S3method(release_selected_all,trajectory)
S3method(renege_abort,trajectory)
S3method(renege_if,trajectory)
S3method(renege_in,trajectory)
S3method(rep,trajectory)
S3method(reset,simmer)
S3method(rollback,trajectory)
S3method(run,simmer)
S3method(seize,trajectory)
S3method(seize_selected,trajectory)
S3method(select,trajectory)
S3method(send,trajectory)
S3method(separate,trajectory)
S3method(set_attribute,trajectory)
S3method(set_capacity,trajectory)
S3method(set_capacity_selected,trajectory)
S3method(set_global,trajectory)
S3method(set_prioritization,trajectory)
S3method(set_queue_size,trajectory)
S3method(set_queue_size_selected,trajectory)
S3method(set_source,trajectory)
S3method(set_trajectory,trajectory)
S3method(stepn,simmer)
S3method(stop_if,trajectory)
S3method(synchronize,trajectory)
S3method(timeout,trajectory)
S3method(timeout_from_attribute,trajectory)
S3method(timeout_from_global,trajectory)
S3method(trap,trajectory)
S3method(untrap,trajectory)
S3method(wait,trajectory)
export("%>%")
export(activate)
export(add_dataframe)
export(add_generator)
export(add_global)
export(add_resource)
export(at)
export(batch)
export(branch)
export(clone)
export(deactivate)
export(from)
export(from_to)
export(get_activity_time)
export(get_activity_time_selected)
export(get_attribute)
export(get_batch_size)
export(get_capacity)
export(get_capacity_selected)
export(get_global)
export(get_mon_arrivals)
export(get_mon_attributes)
export(get_mon_resources)
export(get_n_activities)
export(get_n_generated)
export(get_name)
export(get_prioritization)
export(get_queue_count)
export(get_queue_count_selected)
export(get_queue_size)
export(get_queue_size_selected)
export(get_resources)
export(get_seized)
export(get_seized_selected)
export(get_selected)
export(get_server_count)
export(get_server_count_selected)
export(get_sources)
export(get_start_time)
export(get_trajectory)
export(handle_unfinished)
export(join)
export(leave)
export(log_)
export(monitor)
export(monitor_csv)
export(monitor_delim)
export(monitor_mem)
export(now)
export(peek)
export(release)
export(release_all)
export(release_selected)
export(release_selected_all)
export(renege_abort)
export(renege_if)
export(renege_in)
export(reset)
export(rollback)
export(run)
export(schedule)
export(seize)
export(seize_selected)
export(select)
export(send)
export(separate)
export(set_attribute)
export(set_capacity)
export(set_capacity_selected)
export(set_global)
export(set_prioritization)
export(set_queue_size)
export(set_queue_size_selected)
export(set_source)
export(set_trajectory)
export(simmer)
export(stepn)
export(stop_if)
export(synchronize)
export(timeout)
export(timeout_from_attribute)
export(timeout_from_global)
export(to)
export(trajectory)
export(trap)
export(untrap)
export(wait)
export(when_activated)
export(wrap)
importFrom(Rcpp,evalCpp)
importFrom(codetools,findGlobals)
importFrom(magrittr,"%>%")
importFrom(utils,read.csv)
importFrom(utils,read.delim)
useDynLib(simmer, .registration=TRUE)
